module dix4192updater(clock, reset, sourcesel, rnd, sel4192, dix4192cdout, dix4192cclk, dix4192cdin, dix4192csn);
// clock: 27 MHz sigma-deltaklok, reset: synchrone reset die pas wordt
// weggehaald als de 27 MHz goed loopt.
// clock: 27 MHz sigma-delta clock, reset: synchronized reset that is only
// de-asserted when the 27 MHz is up and running.
input [1:0] sourcesel; // ontdenderd bronselectiesignaal
// debounced source selection signal
input clock, reset, dix4192cdout;
input [5:0] rnd;
input wire sel4192;
output dix4192cdin, dix4192csn;
(* CLOCK_SIGNAL = "TRUE" *) output dix4192cclk;
reg dix4192cdoutt;
reg [2:0] state;
reg [5:0] waitnumber;
reg [4:0] bitcount;
reg [1:0] rxckr;
// geeft aan wat de verhouding tussen de uitgangsfrequentie van PLL2 van de
// DIX4192 en de bemonsteringsfrequentie is, zie het gegevensblad van de DIX4192
// indicates the ratio between the output frequency of PLL2 of the DIX4192 and
// the sample rate, see the DIX4192 datasheet
reg [1:0] rxckod;
// stelt de deler achter PLL2 in, zie het gegevensblad van de DIX4192
// sets the division ratio of the post divider of PLL2, see the DIX4192
// datasheet
reg dix4192csn_int, dix4192csn_intt, dix4192cdin_int;
reg Wim;
(* CLOCK_SIGNAL = "TRUE" *) reg dix4192cclk_int;
reg resetornolock;
wire resetsynch1, resetsynch2, resetv1, resetverlengd;
wire CLK0, CLK180, CLK270, CLK2X, CLK2X180, CLK90, CLKDV, clock3f180, locked, PSDONE;
(* CLOCK_SIGNAL = "TRUE" *) wire clock3f;
wire [7:0] STATUS;
parameter [23:0] const1 = 24'h930000;
// eigenlijk redundante lengtespec, maar werkt wel in simplesequencer
parameter [23:0] const2 = 24'h0E0019; // RXCKO aan, moet dat?
parameter [23:0] const3 = 24'h0D0008;
parameter [23:0] const4 = 24'h1B0000;

always @ *
begin
case (rxckr)
	2'b00: rxckod = 2'b00;
	2'b01: rxckod = 2'b00;
	2'b10: rxckod = 2'b01;
	2'b11: rxckod = 2'b10;
endcase
end

// klokken maken
DCM_SP #(.CLKDV_DIVIDE(3.0), //Afgedeelde uitgang is gerelateerd aan fin en is dus onbruikbaar.
	.CLKFX_DIVIDE(1),
	.CLKFX_MULTIPLY(3),
	.CLKIN_DIVIDE_BY_2("FALSE"),
	.CLKIN_PERIOD(37.037),
	.CLKOUT_PHASE_SHIFT("FIXED"),
	.CLK_FEEDBACK("1X"), //????
	.DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"), //????
	.DFS_FREQUENCY_MODE("LOW"),
	.DLL_FREQUENCY_MODE("LOW"),
	.DSS_MODE("NONE"),
	.DUTY_CYCLE_CORRECTION("TRUE"),
	.FACTORY_JF(16'hc080),
	.PHASE_SHIFT(0), // 0/256 times input clock period
	.STARTUP_WAIT("FALSE"))
	clockmult3 (CLK0, CLK180, CLK270, CLK2X, CLK2X180, CLK90, CLKDV, clock3f, clock3f180, locked, PSDONE, STATUS, CLK0, clock, 1'b0, 1'b0, 1'b0, 1'b0, reset);
	// CLKFB op CLK0 aansluiten is eigenlijk niet eens nodig als je alleen CLKFX maar gebruikt.
// Flip-flops met asynchrone set, flip-flops with asynchronous set
FDPE flipflop3 (resetv1, clock3f, 1'b1, reset, reset);
FDPE flipflop4 (resetverlengd, clock3f, 1'b1, resetv1, resetv1);
always @ (posedge clock3f or posedge resetverlengd)
	if (resetverlengd)
	begin
		dix4192cclk_int <= 1'b0;
		Wim <= 1'b0;
	end
	else
	begin
		case ({dix4192cclk_int, Wim})
			2'b00:
			begin
				dix4192cclk_int <= 1'b0;
				Wim <= 1'b1;
			end
			2'b01:
			begin
				dix4192cclk_int <= 1'b1;
				Wim <= 1'b1;
			end
			default:
			begin
				dix4192cclk_int <= 1'b0;
				Wim <= 1'b0;
			end
		endcase;
	end
always @ * 
begin
	resetornolock = reset || (!locked);
end

// Twee flip-flops met asynchrone set, two flip-flops with asynchronous set
FDPE flipflop1 (resetsynch1, dix4192cclk_int, 1'b1, resetornolock, resetornolock);
FDPE flipflop2 (resetsynch2, dix4192cclk_int, 1'b1, resetsynch1, resetsynch1);

always @ (posedge clock3f) if (!Wim) dix4192csn_int <= dix4192csn_intt;
always @ (posedge clock3f) if (Wim && (!dix4192cclk_int)) dix4192cdoutt <= dix4192cdout;
always @ (posedge clock3f) if (dix4192cclk_int)
begin
	if (resetsynch2)
	begin
		state <= 3'b000;
		waitnumber <= 6'b100000;
		rxckr <= 2'b00;
		bitcount <= 5'b10111;
		dix4192csn_intt <= 1'b1;
	end
	else
	begin
		case (state)
			3'b001: // uitlezen PLL-instelling, reading PLL setting
			begin
				waitnumber <= rnd[5:0];
				if (bitcount == 5'b11111)
					dix4192csn_intt <= 1'b1;
				else
				begin
					dix4192csn_intt <= 1'b0;
					dix4192cdin_int <= const1[bitcount];
				end
				if (bitcount == 5'b00000) rxckr[1] <= dix4192cdoutt;
				if (bitcount == 5'b11111) rxckr[0] <= dix4192cdoutt;
				if (bitcount == 5'b11111)
					state <= 3'b000;
				else
					bitcount <= bitcount - 5'b00001;
			end
			3'b010: // schrijven deelfactor, writing division ratio
			begin
				waitnumber <= rnd[5:0];
				dix4192csn_intt <= 1'b0;
				case (bitcount)
					5'b00010:
						dix4192cdin_int <= rxckod[1];
					5'b00001:
						dix4192cdin_int <= rxckod[0];
					default:
						dix4192cdin_int <= const2[bitcount];
				endcase
				if (bitcount == 5'b00000)
					state <= 3'b000;
				else
					bitcount <= bitcount - 5'b00001;
			end
			3'b011: // schrijven bronselectie, writing source selection
			begin
				waitnumber <= rnd[5:0];
				dix4192csn_intt <= 1'b0;
				case (bitcount)
					5'b00001:
						dix4192cdin_int <= sourcesel[1];
					5'b00000:
						dix4192cdin_int <= sourcesel[0];
					default:
						dix4192cdin_int <= const3[bitcount];
				endcase
				if (bitcount == 5'b00000)
					state <= 3'b000;
				else
					bitcount <= bitcount - 5'b00001;
			end
			3'b100: // instellen GPO1, setting GPO1
// GPO1 van de DIX4192 zit rechtstreeks aan de mute-ingang van de SRC4392 -> laat GPO1 hoog worden bij niet-audio-data wanneer de DIX4192
// geselecteerd is, maak GPO1 laag bij gebruik van de I2S- of ruwe DSD-ingang in plaats van de DIX4192.
// GPO1 of the DIX4192 is directly connected to the mute input of the SRC4392 -> set to flag non-audio data when the DIX4192 is selected,
// set it to 0 when the I2S- or raw DSD-input is used instead of the DIX4192.
			begin
				waitnumber <= rnd[5:0];
				dix4192csn_intt <= 1'b0;
				case (bitcount)
					5'b00010:
						dix4192cdin_int <= sel4192;
					5'b00001:
						dix4192cdin_int <= sel4192;
					default:
						dix4192cdin_int <= const4[bitcount];
				endcase
				if (bitcount == 5'b00000)
					state <= 3'b000;
				else
					bitcount <= bitcount - 5'b00001;
			end
			default: // ruststand, rest state
			begin
				dix4192csn_intt <= 1'b1;
				bitcount <= 5'b10111;
				if (waitnumber == 6'b000000)
					state <= rnd[5:3];
				else
					waitnumber <= waitnumber - 6'b000001;
			end
		endcase
	end
end
assign dix4192cdin = dix4192cdin_int;
assign dix4192csn = (dix4192csn_int && dix4192csn_intt) || resetsynch2;
assign dix4192cclk = dix4192cclk_int;
endmodule
