module lastintegrator3(x, clock, reset, y, clip);
// Integrator zonder vertraging, dus z/(z-1), beschermd tegen overloop,
// maar verder geen begrenzing van de toestandsvariabele.
// Inwendig 1 bit extra, dus eigenlijk een wordlength + 1 bits integrator.
// Integrator without delay, that is z/(z-1), protected against overflow,
// but with no other limiting of the state variable.
// 1 bit extra internally, so actually a wordlength + 1 bit integrator.
	parameter wordlength = 35;
	input signed [wordlength-1:0] x;
	input clock, reset;
	output reg signed [wordlength-1:0] y;
	reg signed [wordlength+1:0] z;
	reg signed [wordlength:0] y_int;
	output clip;
	reg clip_int;
	reg hulp;
	reg signed [wordlength:0] t;
	integer Piet;
    	always @ (posedge clock)
    	begin
		if (reset)
		begin
			t <= 0;
		end
		else
		begin
			t <= y_int;
		end
	end
    	always @ (*)
    	begin
		z = x + t;
		case({z[wordlength+1],z[wordlength],z[wordlength-1]})
			3'b01X:
			begin
				for (Piet=0; Piet<wordlength; Piet=Piet+1)
					y_int[Piet] = 1;
				y_int[wordlength] = 0;
				for (Piet=0; Piet<wordlength-1; Piet=Piet+1)
					y[Piet] = 1;
				y[wordlength-1] = 0;
				clip_int = 1;
			end
			3'b10X:
			begin
				for (Piet=0; Piet<wordlength; Piet=Piet+1)
					y_int[Piet] = 0;
				y_int[wordlength] = 1;
				for (Piet=0; Piet<wordlength-1; Piet=Piet+1)
					y[Piet] = 0;
				y[wordlength-1] = 1;
				clip_int = 1;
			end
			3'b001:
			begin
			    	y_int[wordlength:0] = {z[wordlength+1],z[wordlength-1:0]};
				for (Piet=0; Piet<wordlength-1; Piet=Piet+1)
					y[Piet] = 1;
				y[wordlength-1] = 0;
			    	clip_int = 0;
			end
			3'b110:
			begin
			    	y_int[wordlength:0] = {z[wordlength+1],z[wordlength-1:0]};
				for (Piet=0; Piet<wordlength-1; Piet=Piet+1)
					y[Piet] = 0;
				y[wordlength-1] = 1;
				clip_int = 0;
			end
			default:
			begin
			    	y_int[wordlength:0] = {z[wordlength+1],z[wordlength-1:0]};
			    	y[wordlength-1:0] = {z[wordlength+1],z[wordlength-2:0]};
			   	clip_int = 0;
			end
		endcase
	end
	assign clip=clip_int;
endmodule
