module pwm3(x, m, rot, clock, reset, y);
// pwm-module aangepast voor PBM5 en PBM9, zoals nodig is voor de
// buizen-DAC-configuratie 3 (LX45-variant).
// pwm module modified for PWM5 and PWM9, as required for valve DAC
// configuration 3 (LX45 variant).
	input signed [3:0] x;
	input [1:0] m; // 00: x eenwaardig (single value), 01: tweewaardig (two-valued), 10: zeswaardig (six-valued), 11: tienwaardig (ten-valued)
	input [4:0] rot; // toevalsgetal, 0...31 met elk kans 1/32. random number, 0...31 each with probability 1/32
	input clock, reset;
	output y;
	reg [3:0] hulp;
	reg [3:0] hulp2;
	reg [3:0] hulp3;
	reg [4:0] rot2;
	reg [3:0] rot3;
	reg [3:0] rotation;
	reg y_int;
	reg [3:0] count_int;
	always @ (posedge clock)
	begin
		if (reset)
		begin
			count_int <= 0;
			hulp <= 0;
			rotation <= 0;
		end
		else
		begin
			if (count_int == 0)
			begin
				case(m)
					2'b00:
					begin
						count_int <= 1; // 01010101... voor stilte, 01010101... for silence
						hulp <= 1;
					end
					2'b01:
					begin
						count_int <= 0;
						hulp[3:1] <= 3'b000;
						hulp[0] <= ~x[0];
					end
					2'b10:
					begin
						count_int <= 4;
						hulp[3:0] <= $signed(x[3:0]) + 4'sb0011;
					end
					2'b11:
					begin
						count_int <= 8;
						hulp[3:0] <= $signed(x[3:0]) + 4'sb0101;
					end
				endcase
				rotation <= rot3;
			end
			else
				count_int <= count_int - 4'b0001;
		end
	end
	always @ *
	begin
		hulp2 = count_int + rotation;
		case (m)
			2'b10:
			begin
				if (hulp2 > 4)
					hulp3 = hulp2 - 4'd5;
				else
					hulp3 = hulp2;
			end
			default:
			begin
				if (hulp2 > 8)
					hulp3 = hulp2 - 4'd9;
				else
					hulp3 = hulp2;
			end
		endcase
		if (hulp3 < hulp)
			y_int = 1;
		else
			y_int = 0;
	end
	always @ (posedge clock)
// Converting the 32-level random number rot into a five-level or
// nine-level number for PWM5 and PWM9 mode, respectively.
// The probabilities of each possible number are equal, but a small amount
// of autocorrelation is taken for granted. 
	begin
		case (m)
			2'b00: rot2 <= 5'b00000;
			2'b01: rot2 <= 5'b00000;
			2'b10: rot2 <= {1'b0, rot[3:0]};
			2'b11: rot2 <= rot;
		endcase
		case (rot2[4:0])
			5'd0, 5'd1, 5'd2: rot3 <= 4'd0;
			5'd3, 5'd4, 5'd5: rot3 <= 4'd1;
			5'd6, 5'd7, 5'd8: rot3 <= 4'd2;
			5'd9, 5'd10, 5'd11: rot3 <= 4'd3;
			5'd12, 5'd13, 5'd14: rot3 <= 4'd4;
			5'd16, 5'd17, 5'd18: rot3 <= 4'd5;
			5'd19, 5'd20, 5'd21: rot3 <= 4'd6;
			5'd22, 5'd23, 5'd24: rot3 <= 4'd7;
			5'd25, 5'd26, 5'd27: rot3 <= 4'd8;
			default: rot3 <= rot3;
		endcase
	end
	assign y = y_int;
endmodule
