module quantizer3(x, m, rnd1, rnd2, y, clip);
// Variant voor zes- en tienwaardige kwantisatie
// Vergeleken met de eerdere quantizer is de offsetcorrectie verwijderd, want
// een offset van -0,5 LSB is nu juist gewenst als het bereik van de
// uitgangscodes -3...+2 of -5...+4 is.
// Variant for six- and ten-level quantization
// Offset correction removed, compared to the earlier quantizer, as an offset
// of -0.5 LSB now comes in handy when the output code range is -3...+2 or
// -5...+4.
parameter wordlength = 35;
input signed [wordlength-1:0] x;
input signed [wordlength-4:0] rnd1; //uniform verdeeld, uniform distribution
input signed [wordlength-4:0] rnd2; //uniform verdeeld, uniform distribution
input [1:0] m; //00: uit (off), 01: twee niveaus (two levels), 10: zes niveaus (six levels), 11: tien niveaus (ten levels)
output signed [3:0] y; // Ook bij een eenbitter is 0 dus positief. Also in the single bit case, 0 means positive.
output clip; 
// Iets meer dan half uitgestuurd komt overeen met volle schaal, verder -> clip.
// Driven slightly beyond halfway means full scale, further -> clip.
reg signed [wordlength-3:0] dither;
reg signed [4:0] bla;
reg signed [3:0] y_int;
reg clip_int;
reg signed [wordlength:0] sum;
always @ *
begin
case(m)
	2'b00:
	begin
		dither = 0;
	end
	2'b01:
	begin
		dither = 0;  // 1 bit -> geen ruimte voor dither (no space for dither)
	end
	2'b10:
	begin
		dither = rnd1 + rnd2; //zeswaardig -> volledige dither (six levels -> full dither)
	end
	2'b11:
	begin
		dither = (rnd1>>>1) + (rnd2>>>1); //tienwaardig -> dither bitje zachter (ten levels -> dither one bit softer)
	end
endcase
sum = x+dither;
bla[4:0] = sum[wordlength:wordlength-4];
if ((bla[4:2] == 3'b000)|(bla[4:2] == 3'b111)|((m == 2'b10)&(bla[4:1] == 4'b0010))|((m == 2'b10)&(bla[4:1] == 4'b1101))|((m == 2'b11)&(bla[4:0] == 5'b00100))|((m == 2'b11)&(bla[4:0] == 5'b11011)))
// Voor m = 2 zijn alle uitgangscodes van -3 tot en met +2
// toegestaan en voor m = 3 alle uitgangscodes van -5 tot en met +4.
//
// For m = 2, all output codes from -3 up to and including +2 
// are allowed and for m = 3, all output codes from
// -5 up to and including +4.
//
begin
	clip_int = 0;
	case(m)
		2'b00: y_int[3:0] = 4'b0000;
		2'b01: y_int[3:0] = {bla[3], bla[3], bla[3], bla[3]};
		2'b10: y_int[3:0] = {bla[3], bla[3], bla[2], bla[1]};
		2'b11: y_int[3:0] = bla[3:0];
	endcase
end
else
begin
	clip_int = 1;
	if (bla[4] == 0)
	begin
		case (m)
			2'b00: y_int[3:0] = 4'b0000; 
			2'b01: y_int[3:0] = 4'b0000;
			2'b10: y_int[3:0] = 4'b0010;
			2'b11: y_int[3:0] = 4'b0100;
		endcase
	end
	else
	begin
		case (m)
			2'b00: y_int[3:0] = 4'b0000;
			2'b01: y_int[3:0] = 4'b1111;
			2'b10: y_int[3:0] = 4'b1101;
			2'b11: y_int[3:0] = 4'b1011;
		endcase
	end
end
end
assign y = y_int;
assign clip = clip_int;
endmodule
